@@ -77,7 +77,7 @@
 #' with the source argument in [event_data()] to retrieve the 
 #' event data corresponding to a specific plot (shiny apps can have multiple plots).
 #' @author Carson Sievert
-#' @references <https://plotly-book.cpsievert.me/the-plotly-cookbook.html>
+#' @references <https://plotly-r.com/overview.html>
 #' @seealso \itemize{
 #'  \item For initializing a plotly-geo object: [plot_geo()]
 #'  \item For initializing a plotly-mapbox object: [plot_mapbox()]
@@ -310,11 +310,13 @@ plot_geo <- function(data = data.frame(), ..., offline = FALSE) {
 #' @seealso [plot_ly()], [plot_mapbox()], [ggplotly()] 
 #' @examples
 #' 
+#' \dontrun{
 #' hc <- hclust(dist(USArrests), "ave")
 #' dend1 <- as.dendrogram(hc)
 #' plot_dendro(dend1, height = 600) %>% 
 #'   hide_legend() %>% 
 #'   highlight(persistent = TRUE, dynamic = TRUE)
+#' }
 #' 
 
 plot_dendro <- function(d, set = "A", xmin = -50, height = 500, width = 500, ...) {
@@ -387,14 +389,12 @@ plot_dendro <- function(d, set = "A", xmin = -50, height = 500, width = 500, ...
 }
 
 get_xy <- function(node) {
-  setNames(
-    tibble::as_tibble(dendextend::get_nodes_xy(node)), 
-    c("x", "y")
-  )
+  m <- dendextend::get_nodes_xy(node)
+  colnames(m) <- c("x", "y")
+  tibble::as_tibble(m)
 }
 
 
-
 #' Convert a list to a plotly htmlwidget object
 #' 
 #' @param x a plotly object.
@@ -424,6 +424,15 @@ as_widget <- function(x, ...) {
     ),
     preRenderHook = plotly_build,
     dependencies = c(
+      # phantomjs doesn't support Object.setPrototypeOf() and a
+      # plotly.js dependency (buffer) uses it to detect TypedArray support.
+      # Thus, we add a polyfill if this is running in shinytest, but otherwise
+      # we shouldn't need it because Object.setPrototypeOf() is pretty widely supported
+      # https://github.com/plotly/plotly.js/issues/4556#issuecomment-583061419
+      # https://caniuse.com/#search=setPrototypeOf
+      if (isTRUE(getOption("shiny.testmode"))) {
+        list(setPrototypeOfPolyfill())
+      },
       list(typedArrayPolyfill()),
       crosstalk::crosstalkLibs(),
       list(plotlyHtmlwidgetsCSS()),
@@ -432,22 +441,34 @@ as_widget <- function(x, ...) {
   )
 }
 
+setPrototypeOfPolyfill <- function() {
+  htmltools::htmlDependency(
+    name = "setprototypeof", 
+    version = "0.1",
+    package = "plotly",
+    src = dependency_dir("setprototypeof"),
+    script = "setprototypeof.js",
+    all_files = FALSE
+  )
+}
+
 typedArrayPolyfill <- function() {
   htmltools::htmlDependency(
-    "typedarray", "0.1",
-    src = depPath("typedarray"),
+    name = "typedarray", 
+    version = "0.1",
+    package = "plotly",
+    src = dependency_dir("typedarray"),
     script = "typedarray.min.js",
     all_files = FALSE
   )
 }
 
-# TODO: suggest a plotlyBundles package that has trace-level bundles 
-# and bundle size at print time.
 plotlyMainBundle <- function() {
   htmltools::htmlDependency(
-    "plotly-main", 
-    version = "1.39.2",
-    src = depPath("plotlyjs"),
+    name = "plotly-main", 
+    version = "1.52.2",
+    package = "plotly",
+    src = dependency_dir("plotlyjs"),
     script = "plotly-latest.min.js",
     all_files = FALSE
   )
@@ -455,9 +476,10 @@ plotlyMainBundle <- function() {
 
 plotlyHtmlwidgetsCSS <- function() {
   htmltools::htmlDependency(
-    "plotly-htmlwidgets-css", 
+    name = "plotly-htmlwidgets-css", 
     version = plotlyMainBundle()$version,
-    src = depPath("plotlyjs"),
+    package = "plotly",
+    src = dependency_dir("plotlyjs"),
     stylesheet = "plotly-htmlwidgets.css",
     all_files = FALSE
   )
@@ -468,8 +490,8 @@ locale_dependency <- function(locale) {
     stop("locale must be a character string (vector of length 1)", call. = FALSE)
   }
   
-  locale_dir <- depPath("plotlyjs", "locales")
-  locales_all <- sub("\\.js$", "", list.files(locale_dir))
+  locale_dir <- dependency_dir("plotlyjs", "locales")
+  locales_all <- sub("\\.js$", "", list.files(system.file(locale_dir, package = "plotly")))
   if (!tolower(locale) %in% locales_all) {
     stop(
       "Invalid locale: '", locale, "'.\n\n",
@@ -491,6 +513,7 @@ locale_dependency <- function(locale) {
   htmltools::htmlDependency(
     name = paste0("plotly-locale-", locale),
     version = plotlyMainBundle()$version,
+    package = "plotly",
     src = list(file = locale_dir),
     script = tolower(scripts),
     all_files = FALSE
