@@ -8,3 +8,10 @@ test_that("remove_class() can remove 'AsIs' class", {
   x <- remove_class(I(x), "AsIs")
   expect_equivalent(class(x), c("x", "y"))
 })
+
+
+test_that("default() doesn't change to_JSON() behavior", {
+  x <- 1:10
+  expect_equivalent(to_JSON(default(x)), to_JSON(x))
+})
+
